use std::cell::RefCell;
use std::rc::Rc;

use motor_controller::MockMotorController;
use motor_controller::MotorControllerBox;
use motor_controller::MotorControllerBoxed;

pub struct Drive {
    left_motors: Vec<MotorControllerBox>,
    right_motors: Vec<MotorControllerBox>,
}

pub type DriveRef = Rc<RefCell<Drive>>;

fn create_motor_controllers<T: IntoIterator<Item = u16>>(ids: T) -> Vec<MotorControllerBox> {
    ids.into_iter().map(MockMotorController::new_box).collect()
}

impl Drive {
    pub fn new() -> Self {
        let left_motors = create_motor_controllers(0u16..4);
        let right_motors = create_motor_controllers(4u16..8);
        Drive {
            left_motors,
            right_motors,
        }
    }

    pub fn new_ref() -> DriveRef {
        Rc::new(RefCell::new(Drive::new()))
    }

    pub fn drive_powers(&mut self, left_power: f64, right_power: f64) {
        for motor in &mut self.left_motors {
            motor.set(left_power);
        }
        for motor in &mut self.right_motors {
            motor.set(right_power);
        }
    }
}
