pub trait MotorController {
    fn id(&self) -> u16;
    fn set(&mut self, value: f64);
    fn get(&self) -> f64;
}

pub trait MotorControllerId: MotorController {
    fn new(id: u16) -> Self;
}

pub type MotorControllerBox = Box<dyn MotorController>;

pub trait MotorControllerBoxed {
    fn new_box(id: u16) -> MotorControllerBox;
}

impl<T: MotorControllerId + 'static> MotorControllerBoxed for T {
    fn new_box(id: u16) -> MotorControllerBox {
        Box::new(T::new(id))
    }
}

pub struct MockMotorController {
    id: u16,
    value: f64,
}

impl MotorControllerId for MockMotorController {
    fn new(id: u16) -> Self {
        MockMotorController { id, value: 0.0 }
    }
}

impl MotorController for MockMotorController {
    fn id(&self) -> u16 {
        self.id
    }

    fn set(&mut self, value: f64) {
        self.value = value;
        debug!("MotorController id {} value={}", self.id, self.value);
    }

    fn get(&self) -> f64 {
        self.value
    }
}
